window.Gleam.initSearch([{"doc":"wisp_kv_sessions","title":"wisp_kv_sessions","content":"# wisp_kv_sessions\n\n[![Package Version](https://img.shields.io/hexpm/v/wisp_kv_sessions)](https://hex.pm/packages/wisp_kv_sessions)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/wisp_kv_sessions/)\n\n\n# Overview\nwisp_kv_sessions is a key-value session management library for [Wisp](https://gleam-wisp.github.io/wisp/), inspired by the Rust crate [tower sessions](https://docs.rs/tower-sessions/latest/tower_sessions/#). This library allows you to manage user sessions with ease, storing session data in a simple key-value store. \n\n# Example Usage\nAn minimal example usage is available in `./example/app.gleam`.\n\n```gleam\nimport gleam/erlang/process\nimport gleam/option\nimport gleam/result\nimport gleam/string_builder\nimport mist\nimport wisp\nimport wisp/wisp_mist\nimport wisp_kv_sessions\nimport wisp_kv_sessions/actor_adapter\nimport wisp_kv_sessions/session\nimport wisp_kv_sessions/session_config\n\npub fn main() {\n  // Setup session_adapter\n  use store &lt;- result.map(actor_adapter.new())\n  use cache_store &lt;- result.map(actor_adapter.new())\n\n  // Create session config\n  let session_config =\n    session_config.Config(\n      default_expiry: session.ExpireIn(60 * 60),\n      cookie_name: &quot;SESSION_COOKIE&quot;,\n      store:,\n      cache: option.Some(cache_store),\n    )\n\n  let secret_key_base = wisp.random_string(64)\n\n  // Start the Mist web server.\n  let assert Ok(_) =\n    wisp_mist.handler(handle_request(_, session_config), secret_key_base)\n    |&gt; mist.new\n    |&gt; mist.port(8000)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n\npub fn handle_request(req: wisp.Request, session_config) -&gt; wisp.Response {\n  // Run the middleware and construct current_session\n  use req &lt;- wisp_kv_sessions.middleware(req, session_config)\n  let current_session = wisp_kv_sessions.CurrentSession(req, session_config)\n\n  case wisp.path_segments(req) {\n    [] -&gt; get_value_page(req, current_session)\n    [&quot;set&quot;] -&gt; set_value_page(req, current_session)\n    _ -&gt; wisp.not_found()\n  }\n}\n\nfn get_value_page(\n  _req: wisp.Request,\n  session: wisp_kv_sessions.CurrentSession,\n) -&gt; wisp.Response {\n  // Read value to the session\n  let assert Ok(key) =\n    session\n    |&gt; wisp_kv_sessions.key(&quot;test_key&quot;)\n    |&gt; wisp_kv_sessions.get()\n\n  case key {\n    option.Some(k) -&gt; {\n      wisp.html_response(string_builder.from_string(k), 200)\n    }\n    option.None -&gt; {\n      wisp.html_response(\n        string_builder.from_string(&quot;No value set. Go to /set to set a value&quot;),\n        200,\n      )\n    }\n  }\n}\n\nfn set_value_page(\n  _req: wisp.Request,\n  session: wisp_kv_sessions.CurrentSession,\n) -&gt; wisp.Response {\n  // Set value to the session\n  let _ =\n    session\n    |&gt; wisp_kv_sessions.key(&quot;test_key&quot;)\n    |&gt; wisp_kv_sessions.set(&quot;something&quot;)\n\n  wisp.redirect(&quot;/&quot;)\n}\n```\n\n## Running the Example\n\nTo start the server, cd into /example and run `gleam run`.\n\nVisit (http://localhost:8000) in your browser. The page should display &quot;No value set. Go to /set to set a value.&quot;\n\n\nNavigate to (http://localhost:8000/set). You will be redirected back to the main page, which will now display &quot;something&quot;.\n\n# Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\njust watch-test # Run test and reload on file changes\n```\n\n# Caching\n\nA session config can be passed an optional cache parameter that is a `SessionStore`\nwrapped in an `option.Option`. If the cache is `option.Some` sessions will be \nfetched from the cache. If the data is not in the cache the `store` will be \ntried.\n\nSession data will be automatically added and removed from the cache.\n\n# SessionStore \nThere are different places you may want to store sessions such as \npostgres/ETS/sqlite. The way you integrate with them is through storage adapters \nthat implement the type SessionStore. You can use one of the prebuild storage \nadapters from down below, or implement a new one if the one you\nare looking for does not exist.\n\nFor an example implementation, see `./src/wisp_kv_sessions/ets_adapter.gleam`.\n\n## Included SessionStore adapters\n\n### actor_adapter\nThe actor_adapter driver is suitable for development and testing purposes, \nbut not recommended for production due to its non-concurrent nature. \nInternally, it uses an [actor](https://hexdocs.pm/gleam_otp/gleam/otp/actor.html), \nwhich may become a bottleneck under heavy loads.\n\n*Usage Example:*\n\n```gleam\nimport wisp_kv_sessions/actor_adapter\n\nuse session_store &lt;- result.map(actor_adapter.new())\n\n// ...\n```\nSee `./example/src/app.gleam` for full example\n\n### postgres_adapter\nAlso included is the postgres_adapter, that allows you to use postgres as \nthe storage implementation\n\n```gleam\nimport wisp_kv_sessions/postgres_adapter\n\nlet db = \n  pog.default_config()\n  |&gt; pog.connect()\n\n// Migrate\nuse _ &lt;- result.try(postgres_adapter.migrate_up(conn))\n\n// Setup session_store\nuse session_store &lt;- result.map(postgres_adapter.new(conn))\n\n//...\n```\n\n\n### ets_adapter\n\nThe ets_adapter uses [Erlang Term Storage](https://www.erlang.org/doc/apps/stdlib/ets.html) \nand [carpenter](https://hexdocs.pm/carpenter/) to store session information.\n*This will NOT be persistant after restarts*. But is a good option for caching.\n\n```gleam\nimport wisp_kv_sessions/ets_adapter\n\n// Setup session_store\nuse session_store &lt;- result.map(ets_adapter.new(conn))\n\n//...\n```\n","url":"index.html"},{"doc":"wisp_kv_sessions/session","title":"Expiry","content":"pub type Expiry {\n  ExpireAt(birl.Time)\n  ExpireIn(Int)\n}\n\nExpireAt(birl.Time)\n\n\nExpireIn(Int)\n\n\nSynonyms:\nwisp_kv_sessions/session.Expiry\nwisp_kv_sessions/session Expiry","url":"wisp_kv_sessions/session.html#Expiry"},{"doc":"wisp_kv_sessions/session","title":"Session","content":"pub type Session {\n  Session(\n    id: SessionId,\n    expires_at: #(#(Int, Int, Int), #(Int, Int, Int)),\n    data: dict.Dict(Key, JsonString),\n  )\n}\n\nSession(\n  id: SessionId,\n  expires_at: #(#(Int, Int, Int), #(Int, Int, Int)),\n  data: dict.Dict(Key, JsonString),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session.Session\nwisp_kv_sessions/session Session","url":"wisp_kv_sessions/session.html#Session"},{"doc":"wisp_kv_sessions/session","title":"SessionBuilder","content":"pub type SessionBuilder {\n  SessionBuilder(\n    id: SessionId,\n    expiry: option.Option(Expiry),\n    data: dict.Dict(Key, JsonString),\n  )\n}\n\nSessionBuilder(\n  id: SessionId,\n  expiry: option.Option(Expiry),\n  data: dict.Dict(Key, JsonString),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session.SessionBuilder\nwisp_kv_sessions/session SessionBuilder","url":"wisp_kv_sessions/session.html#SessionBuilder"},{"doc":"wisp_kv_sessions/session","title":"SessionError","content":"pub type SessionError {\n  UnknownError\n  DecodeError\n  DbSetupError\n  DbErrorInsertError(String)\n  DbErrorGetError(String)\n  DbErrorDeleteError(String)\n  DeserializeError(String)\n  SessionExpiredError\n  NoSessionError\n}\n\nUnknownError\n\n\nDecodeError\n\n\nDbSetupError\n\n\nDbErrorInsertError(String)\n\n\nDbErrorGetError(String)\n\n\nDbErrorDeleteError(String)\n\n\nDeserializeError(String)\n\n\nSessionExpiredError\n\n\nNoSessionError\n\n\nSynonyms:\nwisp_kv_sessions/session.SessionError\nwisp_kv_sessions/session SessionError","url":"wisp_kv_sessions/session.html#SessionError"},{"doc":"wisp_kv_sessions/session","title":"SessionId","content":"pub opaque type SessionId\n\n\nSynonyms:\nwisp_kv_sessions/session.SessionId\nwisp_kv_sessions/session SessionId","url":"wisp_kv_sessions/session.html#SessionId"},{"doc":"wisp_kv_sessions/session","title":"build","content":"pub fn build(session: SessionBuilder) -&gt; Session\n\nSynonyms:\nwisp_kv_sessions/session.build\nwisp_kv_sessions/session build","url":"wisp_kv_sessions/session.html#build"},{"doc":"wisp_kv_sessions/session","title":"builder","content":"pub fn builder() -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.builder\nwisp_kv_sessions/session builder","url":"wisp_kv_sessions/session.html#builder"},{"doc":"wisp_kv_sessions/session","title":"builder_from","content":"pub fn builder_from(session: Session) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.builder_from\nwisp_kv_sessions/session builder_from","url":"wisp_kv_sessions/session.html#builder_from"},{"doc":"wisp_kv_sessions/session","title":"expiry_to_date","content":"pub fn expiry_to_date(\n  expiry: Expiry,\n) -&gt; #(#(Int, Int, Int), #(Int, Int, Int))\n Convert expiry type to a date\nSynonyms:\nwisp_kv_sessions/session.expiry_to_date\nwisp_kv_sessions/session expiry_to_date","url":"wisp_kv_sessions/session.html#expiry_to_date"},{"doc":"wisp_kv_sessions/session","title":"generate_id","content":"pub fn generate_id() -&gt; SessionId\n Create a new session id\nSynonyms:\nwisp_kv_sessions/session.generate_id\nwisp_kv_sessions/session generate_id","url":"wisp_kv_sessions/session.html#generate_id"},{"doc":"wisp_kv_sessions/session","title":"id_from_string","content":"pub fn id_from_string(id: String) -&gt; SessionId\n Unwrap session id to a string\nSynonyms:\nwisp_kv_sessions/session.id_from_string\nwisp_kv_sessions/session id_from_string","url":"wisp_kv_sessions/session.html#id_from_string"},{"doc":"wisp_kv_sessions/session","title":"id_to_string","content":"pub fn id_to_string(id: SessionId) -&gt; String\n Unwrap session id to a string\nSynonyms:\nwisp_kv_sessions/session.id_to_string\nwisp_kv_sessions/session id_to_string","url":"wisp_kv_sessions/session.html#id_to_string"},{"doc":"wisp_kv_sessions/session","title":"with_data","content":"pub fn with_data(\n  session: SessionBuilder,\n  data: Dict(String, String),\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_data\nwisp_kv_sessions/session with_data","url":"wisp_kv_sessions/session.html#with_data"},{"doc":"wisp_kv_sessions/session","title":"with_entry","content":"pub fn with_entry(\n  session: SessionBuilder,\n  key: String,\n  data: String,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_entry\nwisp_kv_sessions/session with_entry","url":"wisp_kv_sessions/session.html#with_entry"},{"doc":"wisp_kv_sessions/session","title":"with_expires_at","content":"pub fn with_expires_at(\n  session: SessionBuilder,\n  expires_at: Time,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_expires_at\nwisp_kv_sessions/session with_expires_at","url":"wisp_kv_sessions/session.html#with_expires_at"},{"doc":"wisp_kv_sessions/session","title":"with_expiry","content":"pub fn with_expiry(\n  session: SessionBuilder,\n  expiry: Expiry,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_expiry\nwisp_kv_sessions/session with_expiry","url":"wisp_kv_sessions/session.html#with_expiry"},{"doc":"wisp_kv_sessions/session","title":"with_id","content":"pub fn with_id(\n  session: SessionBuilder,\n  id: SessionId,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_id\nwisp_kv_sessions/session with_id","url":"wisp_kv_sessions/session.html#with_id"},{"doc":"wisp_kv_sessions/session","title":"with_id_string","content":"pub fn with_id_string(\n  session: SessionBuilder,\n  id: String,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_id_string\nwisp_kv_sessions/session with_id_string","url":"wisp_kv_sessions/session.html#with_id_string"},{"doc":"wisp_kv_sessions/session","title":"wisp_kv_sessions/session","content":"","url":"wisp_kv_sessions/session.html"},{"doc":"wisp_kv_sessions/session_config","title":"Config","content":"pub type Config {\n  Config(\n    default_expiry: session.Expiry,\n    cookie_name: String,\n    store: SessionStore,\n    cache: option.Option(SessionStore),\n  )\n}\n\nConfig(\n  default_expiry: session.Expiry,\n  cookie_name: String,\n  store: SessionStore,\n  cache: option.Option(SessionStore),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session_config.Config\nwisp_kv_sessions/session_config Config","url":"wisp_kv_sessions/session_config.html#Config"},{"doc":"wisp_kv_sessions/session_config","title":"SessionStore","content":"pub type SessionStore {\n  SessionStore(\n    get_session: fn(session.SessionId) -&gt;\n      Result(\n        option.Option(session.Session),\n        session.SessionError,\n      ),\n    save_session: fn(session.Session) -&gt;\n      Result(session.Session, session.SessionError),\n    delete_session: fn(session.SessionId) -&gt;\n      Result(Nil, session.SessionError),\n  )\n}\n Session store is what the different storages should implement.\n It is used when saving and getting session\n\nSessionStore(\n  get_session: fn(session.SessionId) -&gt;\n    Result(option.Option(session.Session), session.SessionError),\n  save_session: fn(session.Session) -&gt;\n    Result(session.Session, session.SessionError),\n  delete_session: fn(session.SessionId) -&gt;\n    Result(Nil, session.SessionError),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session_config.SessionStore\nwisp_kv_sessions/session_config SessionStore","url":"wisp_kv_sessions/session_config.html#SessionStore"},{"doc":"wisp_kv_sessions/session_config","title":"wisp_kv_sessions/session_config","content":"","url":"wisp_kv_sessions/session_config.html"},{"doc":"wisp_kv_sessions","title":"CurrentSession","content":"pub type CurrentSession {\n  CurrentSession(\n    req: wisp.Request,\n    config: session_config.Config,\n  )\n}\n\nCurrentSession(req: wisp.Request, config: session_config.Config)\n\n\nSynonyms:\nwisp_kv_sessions.CurrentSession\nwisp_kv_sessions CurrentSession","url":"wisp_kv_sessions.html#CurrentSession"},{"doc":"wisp_kv_sessions","title":"SessionKey","content":"pub type SessionKey(data) {\n  SessionKey(\n    current_session: CurrentSession,\n    key: String,\n    decode: Decoder(data),\n    encode: fn(data) -&gt; String,\n  )\n}\n\nSessionKey(\n  current_session: CurrentSession,\n  key: String,\n  decode: Decoder(data),\n  encode: fn(data) -&gt; String,\n)\n\n\nSynonyms:\nwisp_kv_sessions.SessionKey\nwisp_kv_sessions SessionKey","url":"wisp_kv_sessions.html#SessionKey"},{"doc":"wisp_kv_sessions","title":"delete","content":"pub fn delete(\n  current_session: CurrentSession,\n  key: String,\n) -&gt; Result(Session, SessionError)\n Delete key from session\n\nSynonyms:\nwisp_kv_sessions.delete\nwisp_kv_sessions delete","url":"wisp_kv_sessions.html#delete"},{"doc":"wisp_kv_sessions","title":"delete_session","content":"pub fn delete_session(\n  current_session: CurrentSession,\n) -&gt; Result(Nil, SessionError)\n Remove session\n Usage: \n \n sessions.delete(store, req)\n \nSynonyms:\nwisp_kv_sessions.delete_session\nwisp_kv_sessions delete_session","url":"wisp_kv_sessions.html#delete_session"},{"doc":"wisp_kv_sessions","title":"get","content":"pub fn get(\n  entry: SessionKey(a),\n) -&gt; Result(Option(a), SessionError)\n Get data from session by key\n\nSynonyms:\nwisp_kv_sessions.get\nwisp_kv_sessions get","url":"wisp_kv_sessions.html#get"},{"doc":"wisp_kv_sessions","title":"get_session","content":"pub fn get_session(\n  current_session: CurrentSession,\n) -&gt; Result(Session, SessionError)\n Try to get the session from the store. \n If it does not exist create a new one.\n\nSynonyms:\nwisp_kv_sessions.get_session\nwisp_kv_sessions get_session","url":"wisp_kv_sessions.html#get_session"},{"doc":"wisp_kv_sessions","title":"key","content":"pub fn key(\n  current_session: CurrentSession,\n  key: String,\n) -&gt; SessionKey(String)\n\nSynonyms:\nwisp_kv_sessions.key\nwisp_kv_sessions key","url":"wisp_kv_sessions.html#key"},{"doc":"wisp_kv_sessions","title":"middleware","content":"pub fn middleware(\n  req: Request(Connection),\n  config: Config,\n  handle_request: fn(Request(Connection)) -&gt; Response(Body),\n) -&gt; Response(Body)\n Create a session if no session exists to make sure there \n always is a session_id to store data towards\n Usage: \n \n use &lt;- sessions.middleware(current_session)\n \nSynonyms:\nwisp_kv_sessions.middleware\nwisp_kv_sessions middleware","url":"wisp_kv_sessions.html#middleware"},{"doc":"wisp_kv_sessions","title":"replace_session","content":"pub fn replace_session(\n  current_session: CurrentSession,\n  res: Response(Body),\n  new_session: Session,\n) -&gt; Result(Response(Body), SessionError)\n Replace the session with a new one\n Usage:\n \n wisp.ok() |&gt; replace_session(new_session)\n \n\nSynonyms:\nwisp_kv_sessions.replace_session\nwisp_kv_sessions replace_session","url":"wisp_kv_sessions.html#replace_session"},{"doc":"wisp_kv_sessions","title":"set","content":"pub fn set(\n  entry: SessionKey(a),\n  data: a,\n) -&gt; Result(a, SessionError)\n Set data in session by key\n\nSynonyms:\nwisp_kv_sessions.set\nwisp_kv_sessions set","url":"wisp_kv_sessions.html#set"},{"doc":"wisp_kv_sessions","title":"with_codec","content":"pub fn with_codec(\n  session_key: SessionKey(a),\n  decoder decode: fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n  encoder encode: fn(b) -&gt; String,\n) -&gt; SessionKey(b)\n\nSynonyms:\nwisp_kv_sessions.with_codec\nwisp_kv_sessions with_codec","url":"wisp_kv_sessions.html#with_codec"},{"doc":"wisp_kv_sessions","title":"wisp_kv_sessions","content":"","url":"wisp_kv_sessions.html"},{"doc":"wisp_kv_sessions/actor_adapter","title":"new","content":"pub fn new() -&gt; Result(SessionStore, SessionError)\n\nSynonyms:\nwisp_kv_sessions/actor_adapter.new\nwisp_kv_sessions/actor_adapter new","url":"wisp_kv_sessions/actor_adapter.html#new"},{"doc":"wisp_kv_sessions/actor_adapter","title":"wisp_kv_sessions/actor_adapter","content":"","url":"wisp_kv_sessions/actor_adapter.html"},{"doc":"wisp_kv_sessions/ets_adapter","title":"new","content":"pub fn new(table_name: String) -&gt; SessionStore\n\nSynonyms:\nwisp_kv_sessions/ets_adapter.new\nwisp_kv_sessions/ets_adapter new","url":"wisp_kv_sessions/ets_adapter.html#new"},{"doc":"wisp_kv_sessions/ets_adapter","title":"new_table","content":"pub fn new_table(table_name: String) -&gt; Set(String, Session)\n\nSynonyms:\nwisp_kv_sessions/ets_adapter.new_table\nwisp_kv_sessions/ets_adapter new_table","url":"wisp_kv_sessions/ets_adapter.html#new_table"},{"doc":"wisp_kv_sessions/ets_adapter","title":"wisp_kv_sessions/ets_adapter","content":"","url":"wisp_kv_sessions/ets_adapter.html"},{"doc":"wisp_kv_sessions/postgres_adapter","title":"migrate_down","content":"pub fn migrate_down(\n  db: Connection,\n) -&gt; Result(Returned(Nil), QueryError)\n\nSynonyms:\nwisp_kv_sessions/postgres_adapter.migrate_down\nwisp_kv_sessions/postgres_adapter migrate_down","url":"wisp_kv_sessions/postgres_adapter.html#migrate_down"},{"doc":"wisp_kv_sessions/postgres_adapter","title":"migrate_up","content":"pub fn migrate_up(\n  db: Connection,\n) -&gt; Result(Returned(Nil), QueryError)\n\nSynonyms:\nwisp_kv_sessions/postgres_adapter.migrate_up\nwisp_kv_sessions/postgres_adapter migrate_up","url":"wisp_kv_sessions/postgres_adapter.html#migrate_up"},{"doc":"wisp_kv_sessions/postgres_adapter","title":"new","content":"pub fn new(db: Connection) -&gt; SessionStore\n\nSynonyms:\nwisp_kv_sessions/postgres_adapter.new\nwisp_kv_sessions/postgres_adapter new","url":"wisp_kv_sessions/postgres_adapter.html#new"},{"doc":"wisp_kv_sessions/postgres_adapter","title":"wisp_kv_sessions/postgres_adapter","content":"","url":"wisp_kv_sessions/postgres_adapter.html"}]);